/*
 * Copyright (C) 2013 Google Inc. All Rights Reserved. 
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at 
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 *
 * Modified by Octoshape 2015
 */

package com.google.sample.cast.refplayer.browser;

import android.app.ProgressDialog;
import android.os.Bundle;
import android.support.v4.app.ListFragment;
import android.support.v4.app.LoaderManager;
import android.support.v4.content.Loader;
import android.util.Log;
import android.view.View;
import android.widget.ListView;
import android.widget.Toast;

import com.google.android.gms.cast.MediaInfo;
import com.google.android.libraries.cast.companionlibrary.utils.Utils;

import octoshape.android.CastApplication;
import octoshape.android.R;

import java.util.List;

import octoshape.android.stream.IStreamPlayerWrapperListener;
import octoshape.android.stream.StreamManager;

public class StreamListFragment extends ListFragment implements
        LoaderManager.LoaderCallbacks<List<MediaInfo>> {

    private static final String TAG = "StreamListFragment";
    private StreamListAdapter mAdapter;
    private ProgressDialog mLoading;

    /*
     * (non-Javadoc)
     * @see android.support.v4.app.Fragment#onActivityCreated(android.os.Bundle)
     */
    @Override
    public void onActivityCreated(Bundle savedInstanceState) {
        super.onActivityCreated(savedInstanceState);
        getListView().setFastScrollEnabled(true);
        mAdapter = new StreamListAdapter(getActivity());

        setEmptyText(getString(R.string.no_video_found));
        setListAdapter(mAdapter);
        setListShown(false);
        getLoaderManager().initLoader(0, null, this);
    }

    /*
     * (non-Javadoc)
     * @see android.support.v4.app.LoaderManager.LoaderCallbacks#onLoadFinished(android
     * .support.v4.content.Loader, java.lang.Object)
     */
    @Override
    public void onLoadFinished(Loader<List<MediaInfo>> arg0, List<MediaInfo> data) {
        mAdapter.setData(data);
        if (isResumed()) {
            setListShown(true);
        } else {
            setListShownNoAnimation(true);
        }
    }

    /*
     * (non-Javadoc)
     * @see android.support.v4.app.LoaderManager.LoaderCallbacks#onLoaderReset(android
     * .support.v4.content.Loader)
     */
    @Override
    public void onLoaderReset(Loader<List<MediaInfo>> arg0) {
        mAdapter.setData(null);
    }

    @Override
    public void onListItemClick(ListView l, View v, final int position, long id) {
        final MediaInfo selectedMedia = mAdapter.getItem(position);

        // in case there was another remote player, stop playing.
        CastApplication.getPlayerManager().stopPlaying();

        mLoading = ProgressDialog.show(getActivity(), "Loading stream", "Please wait ...", true);
        mLoading.setCancelable(false);

        if (CastApplication.getStreamManager().isConnected()) {
            // setup the stream and register a listener
            CastApplication.getStreamManager().setupStream(
                    selectedMedia,
                    new IStreamPlayerWrapperListener() {
                        boolean hasAskedToPlay = false;

                        public void streamConnected(final MediaInfo mediaInfo) {
                            Log.d(TAG, "Will start playing stream: " + StreamManager.getStreamFromMediaInfo(mediaInfo));
                            if (mLoading != null) {
                                mLoading.dismiss();
                                mLoading = null;
                            }

                            // Once the stream is connected, and this is not just an update,
                            // start playing
                            if (!hasAskedToPlay) {
                                CastApplication.getPlayerManager().startPlaying(StreamListFragment.this.getActivity(), Utils.mediaInfoToBundle(mediaInfo));
                            }

                            hasAskedToPlay = true;
                        }

                        public void gotProblem(final String message) {

                            if (mLoading != null) {
                                mLoading.dismiss();
                                mLoading = null;
                            }
                            showError(message);
                        }

                    });
        } else {
            Toast.makeText(getActivity(), "Still connecting to OctoshapeSystem", Toast.LENGTH_LONG).show();
        }
    }

    private void showError(final String errorMessage) {
        getActivity().runOnUiThread(new Runnable() {
            @Override
            public void run() {
                Toast.makeText(getActivity(), errorMessage, Toast.LENGTH_SHORT).show();
            }
        });
    }

    /*
     * (non-Javadoc)
     * @see android.support.v4.app.LoaderManager.LoaderCallbacks#onCreateLoader(int,
     * android.os.Bundle)
     */
    @Override
    public Loader<List<MediaInfo>> onCreateLoader(int arg0, Bundle arg1) {
        return new StreamItemLoader(getActivity().getApplicationContext());
    }
}